//
//  ConversationMainStatusTableViewCell.swift
//  Tusker
//
//  Created by Shadowfacts on 8/28/18.
//  Copyright © 2018 Shadowfacts. All rights reserved.
//

import UIKit
import Combine
import Pachyderm

class ConversationMainStatusTableViewCell: UITableViewCell {

    static let dateFormatter: DateFormatter = {
        let formatter = DateFormatter()
        formatter.dateStyle = .medium
        formatter.timeStyle = .medium
        return formatter
    }()
    
    var delegate: StatusTableViewCellDelegate? {
        didSet {
            contentLabel.navigationDelegate = delegate
        }
    }
    
    @IBOutlet weak var profileDetailContainerView: UIView!
    @IBOutlet weak var avatarImageView: UIImageView!
    @IBOutlet weak var displayNameLabel: UILabel!
    @IBOutlet weak var usernameLabel: UILabel!
    @IBOutlet weak var contentWarningLabel: UILabel!
    @IBOutlet weak var collapseButton: UIButton!
    @IBOutlet weak var contentLabel: StatusContentLabel!
    @IBOutlet weak var favoriteAndReblogCountStackView: UIStackView!
    @IBOutlet weak var totalFavoritesButton: UIButton!
    @IBOutlet weak var totalReblogsButton: UIButton!
    @IBOutlet weak var timestampAndClientLabel: UILabel!
    @IBOutlet weak var attachmentsView: AttachmentsContainerView!
    @IBOutlet weak var replyButton: UIButton!
    @IBOutlet weak var favoriteButton: UIButton!
    @IBOutlet weak var reblogButton: UIButton!
    @IBOutlet weak var moreButton: UIButton!
    
    var profileAccessibilityElement: UIAccessibilityElement!
    
    var statusID: String!
    var accountID: String!
    
    var favorited: Bool = false {
        didSet {
            DispatchQueue.main.async {
                self.favoriteButton.tintColor = self.favorited ? UIColor(displayP3Red: 1, green: 0.8, blue: 0, alpha: 1) : self.tintColor
            }
        }
    }
    var reblogged: Bool = false {
        didSet {
            DispatchQueue.main.async {
                self.reblogButton.tintColor = self.reblogged ? UIColor(displayP3Red: 1, green: 0.8, blue: 0, alpha: 1) : self.tintColor
            }
        }
    }
    
    var collapsible = false {
        didSet {
            collapseButton.isHidden = !collapsible
        }
    }
    var collapsed = false
    
    var avatarURL: URL?

    var statusUpdater: Cancellable?
    var accountUpdater: Cancellable?
    
    deinit {
        statusUpdater?.cancel()
        accountUpdater?.cancel()
    }
    
    override func awakeFromNib() {
        displayNameLabel.addGestureRecognizer(UITapGestureRecognizer(target: self, action: #selector(accountPressed)))
        usernameLabel.addGestureRecognizer(UITapGestureRecognizer(target: self, action: #selector(accountPressed)))
        avatarImageView.addGestureRecognizer(UITapGestureRecognizer(target: self, action: #selector(accountPressed)))
        avatarImageView.layer.masksToBounds = true
        attachmentsView.delegate = self
        attachmentsView.layer.cornerRadius = 5
        attachmentsView.layer.masksToBounds = true
        collapseButton.layer.masksToBounds = true
        collapseButton.layer.cornerRadius = 5
        
        NotificationCenter.default.addObserver(self, selector: #selector(updateUIForPreferences), name: .preferencesChanged, object: nil)
        
        profileAccessibilityElement = UIAccessibilityElement(accessibilityContainer: self)
        profileAccessibilityElement.accessibilityFrameInContainerSpace = profileDetailContainerView.convert(profileDetailContainerView.frame, to: self)
        accessibilityElements = [profileAccessibilityElement!, contentWarningLabel!, collapseButton!, contentLabel!, totalFavoritesButton!, totalReblogsButton!, timestampAndClientLabel!, replyButton!, favoriteButton!, reblogButton!, moreButton!]
        
        statusUpdater = MastodonCache.statusSubject
            .filter { $0.id == self.statusID }
            .receive(on: DispatchQueue.main)
            .sink(receiveValue: updateStatusState(status:))
        
        accountUpdater = MastodonCache.accountSubject
            .filter { $0.id == self.accountID }
            .receive(on: DispatchQueue.main)
            .sink(receiveValue: updateUI(account:))
    }
    
    func updateUI(statusID: String) {
        guard let status = MastodonCache.status(for: statusID) else { fatalError("Missing cached status \(statusID)") }
        
        self.statusID = status.id
        
        let account: Account
        if let reblog = status.reblog {
            account = reblog.account
        } else {
            account = status.account
        }
        self.accountID = account.id
        updateUI(account: account)
        updateUIForPreferences()
        
        var timestampAndClientText = ConversationMainStatusTableViewCell.dateFormatter.string(from: status.createdAt)
        if let application = status.application {
            timestampAndClientText += " • \(application.name)"
        }
        timestampAndClientLabel.text = timestampAndClientText
        
        attachmentsView.updateUI(status: status)

        let realStatus = status.reblog ?? status
        updateStatusState(status: realStatus)

        contentLabel.statusID = statusID
        
        collapsible = !status.spoilerText.isEmpty
        setCollapsed(collapsible, animated: false)
        contentWarningLabel.text = status.spoilerText
        contentWarningLabel.isHidden = status.spoilerText.isEmpty
    }
    
    private func updateStatusState(status: Status) {
        favorited = status.favourited ?? false
        reblogged = status.reblogged ?? false
        
        if favorited {
            favoriteButton.accessibilityLabel = NSLocalizedString("Undo Favorite", comment: "undo favorite button accessibility label")
        } else {
            favoriteButton.accessibilityLabel = NSLocalizedString("Favorite", comment: "favorite button accessibility label")
        }
        if reblogged {
            reblogButton.accessibilityLabel = NSLocalizedString("Undo Reblog", comment: "undo reblog button accessibility label")
        } else {
            reblogButton.accessibilityLabel = NSLocalizedString("Reblog", comment: "reblog button accessibility label")
        }
        
        // todo: localize me
        totalFavoritesButton.setTitle("\(status.favouritesCount) Favorite\(status.favouritesCount == 1 ? "" : "s")", for: .normal)
        totalReblogsButton.setTitle("\(status.reblogsCount) Reblog\(status.reblogsCount == 1 ? "" : "s")", for: .normal)
    }
    
    private func updateUI(account: Account) {
        usernameLabel.text = "@\(account.acct)"
        avatarImageView.image = nil
        avatarURL = account.avatar
        ImageCache.avatars.get(account.avatar) { (data) in
            guard let data = data else { return }
            DispatchQueue.main.async {
                self.avatarImageView.image = UIImage(data: data)
                self.avatarURL = nil
            }
        }
        
        profileAccessibilityElement.accessibilityLabel = account.realDisplayName
    }
    
    @objc func updateUIForPreferences() {
        guard let account = MastodonCache.account(for: accountID) else { fatalError("Missing cached account \(accountID!)") }
        
        avatarImageView.layer.cornerRadius = Preferences.shared.avatarStyle.cornerRadius(for: avatarImageView)
        displayNameLabel.text = account.realDisplayName
        favoriteAndReblogCountStackView.isHidden = !Preferences.shared.showFavoriteAndReblogCounts
    }

    override func prepareForReuse() {
        if let url = avatarURL {
            ImageCache.avatars.cancel(url)
        }
        attachmentsView.subviews.forEach { $0.removeFromSuperview() }
    }
    
    @objc func accountPressed() {
        delegate?.selected(account: accountID)
    }

    @IBAction func collapsePressed(_ sender: Any) {
        setCollapsed(!collapsed, animated: true)
        delegate?.statusCollapsedStateChanged()
    }
    
    func setCollapsed(_ collapsed: Bool, animated: Bool) {
        self.collapsed = collapsed
        
        contentLabel.isHidden = collapsed
        attachmentsView.isHidden = attachmentsView.attachments.count == 0 || collapsed
        
        let buttonImage = UIImage(systemName: collapsed ? "chevron.down" : "chevron.up")
        
        if animated, let buttonImageView = collapseButton.imageView {
            // see comment in StatusTableViewCell.setCollapsed
            UIView.animateKeyframes(withDuration: 0.2, delay: 0, options: .calculationModeLinear, animations: {
                UIView.addKeyframe(withRelativeStartTime: 0, relativeDuration: 0.5) {
                    buttonImageView.transform = CGAffineTransform(rotationAngle: collapsed ? .pi / 2 : -.pi / 2)
                }
                UIView.addKeyframe(withRelativeStartTime: 0.5, relativeDuration: 0.5) {
                    buttonImageView.transform = CGAffineTransform(rotationAngle: .pi)
                }
            }, completion: { (finished) in
                buttonImageView.transform = .identity
                self.collapseButton.setImage(buttonImage, for: .normal)
            })
        } else {
            collapseButton.setImage(buttonImage, for: .normal)
        }
        
        
        if collapsed {
            collapseButton.accessibilityLabel = NSLocalizedString("Expand Status", comment: "expand status button accessibility label")
        } else {
            collapseButton.accessibilityLabel = NSLocalizedString("Collapse Status", comment: "collapse status button accessibility label")
        }
    }
    
    @IBAction func replyPressed(_ sender: Any) {
        delegate?.reply(to: statusID)
    }
    
    @IBAction func favoritePressed(_ sender: Any) {
        guard let status = MastodonCache.status(for: statusID) else { fatalError("Missing cached status \(statusID!)") }
        
        favorited = !favorited
        
        let realStatus: Status = status.reblog ?? status
        
        let request = (favorited ? Status.favourite : Status.unfavourite)(realStatus)
        MastodonController.client.run(request) { response in
            DispatchQueue.main.async {
                if case let .success(newStatus, _) = response {
                    self.favorited = newStatus.favourited ?? false
                    MastodonCache.add(status: newStatus)
                    UIImpactFeedbackGenerator(style: .light).impactOccurred()
                } else {
                    print("Couldn't favorite status \(realStatus.id)")
                    // todo: display error message
                    UINotificationFeedbackGenerator().notificationOccurred(.error)
                    return
                }
            }
        }
    }
    
    @IBAction func reblogPressed(_ sender: Any) {
        guard let status = MastodonCache.status(for: statusID) else { fatalError("Missing cached status \(statusID!)") }
        
        reblogged = !reblogged
        
        let realStatus: Status = status.reblog ?? status
        
        let request = (reblogged ? Status.reblog : Status.unreblog)(realStatus)
        MastodonController.client.run(request) { response in
            DispatchQueue.main.async {
                if case let .success(newStatus, _) = response {
                    self.reblogged = newStatus.reblogged ?? false
                    MastodonCache.add(status: newStatus)
                    UIImpactFeedbackGenerator(style: .light).impactOccurred()
                } else {
                    print("Couldn't reblog status \(realStatus.id)")
                    // todo: display error message
                    UINotificationFeedbackGenerator().notificationOccurred(.error)
                }
            }
        }
    }
    
    @IBAction func morePressed(_ sender: Any) {
        delegate?.showMoreOptions(forStatus: statusID)
    }
    
    @IBAction func totalFavoritesPressed(_ sender: Any) {
        if let delegate = delegate {
            // accounts aren't known, pass nil so the VC will load them
            let vc = delegate.statusActionAccountList(action: .favorite, statusID: statusID, accountIDs: nil)
            vc.showInacurateCountWarning = true
            delegate.show(vc)
        }
    }
    
    @IBAction func totalReblogsPressed(_ sender: Any) {
        if let delegate = delegate {
            // accounts aren't known, pass nil so the VC will load them
            let vc = delegate.statusActionAccountList(action: .reblog, statusID: statusID, accountIDs: nil)
            vc.showInacurateCountWarning = true
            delegate.show(vc)
        }
    }
}

extension ConversationMainStatusTableViewCell: AttachmentViewDelegate {
    func showAttachmentsGallery(startingAt index: Int) {
        guard let status = MastodonCache.status(for: statusID) else { fatalError("Missing cached status \(statusID!)") }
        let sourceViews = status.attachments.map(attachmentsView.getAttachmentView(for:))
        delegate?.showGallery(attachments: status.attachments, sourceViews: sourceViews, startIndex: index)
    }
}

extension ConversationMainStatusTableViewCell: MenuPreviewProvider {
    func getPreviewProviders(for location: CGPoint, sourceViewController: UIViewController) -> PreviewProviders? {
        if avatarImageView.frame.contains(location) {
            return (content: { ProfileTableViewController(accountID: self.accountID) }, actions: { self.actionsForProfile(accountID: self.accountID) })
        } else if attachmentsView.frame.contains(location) {
            let attachmentsViewLocation = attachmentsView.convert(location, from: self)
            if let attachmentView = attachmentsView.subviews.first(where: { $0.frame.contains(attachmentsViewLocation) }) as? AttachmentView {
                let image = attachmentView.image!
                let description = attachmentView.attachment.description
                return (content: { self.delegate?.largeImage(image, description: description, sourceView: attachmentView) }, actions: { [] })
            }
        } else if contentLabel.frame.contains(location),
            let link = contentLabel.getLink(atPoint: contentLabel.convert(location, from: self)) {
            return (
                content: { self.contentLabel.getViewController(forLink: link.url, inRange: link.range) },
                actions: {
                    let text = (self.contentLabel.text! as NSString).substring(with: link.range)
                    if let mention = self.contentLabel.getMention(for: link.url, text: text) {
                        return self.actionsForProfile(accountID: mention.id)
                    } else if let hashtag = self.contentLabel.getHashtag(for: link.url, text: text) {
                        return self.actionsForHashtag(hashtag)
                    } else {
                        return self.actionsForURL(link.url)
                    }
                }
            )
        }
        return nil
    }
}
